@@ -6,36 +6,30 @@ use Carp;
 
 use Coro;
 use Coro::Handle qw(unblock);
-use Coro::Semaphore;
 use Coro::ProcessPool::Util qw(encode decode $EOL);
 
+use fields qw(
+    counter
+    in
+    out
+    inbox
+    inbox_mon
+    inbox_running
+);
+
 sub new {
     my ($class, $fh_in, $fh_out) = @_;
+    my $self = fields::new($class);
 
-    my $self = bless {
-        counter       => 0,
-        in            => unblock $fh_in,
-        out           => unblock $fh_out,
-        inbox         => {},
-        read_sem      => Coro::Semaphore->new(0),
-        inbox_running => 1,
-    }, $class;
+    $self->{counter} = 0;
+    $self->{in}      = unblock $fh_in;
+    $self->{out}     = unblock $fh_out;
+    $self->{inbox}   = {};
 
+    $self->{inbox_running} = 1;
     $self->{inbox_mon} = async {
-        while (1) {
-            $self->{in}->readable or last;
-
-            # If anyone is waiting on this inbox to have data, wake them up and
-            # cede control to them. This is used in Coro::ProcessPool::process
-            # to put the worker back into the queue as soon as the result is
-            # ready, rather than waiting until the result is completely read.
-            my $waiting = $self->{read_sem}->waiters;
-            if ($waiting > 0) {
-              $self->{read_sem}->adjust($waiting);
-              cede;
-            }
-
-            my $line = $self->{in}->readline($EOL) or last;
+        while (my $line = $self->{in}->readline($EOL)) {
+            last unless $line;
             my $msg = decode($line);
             my ($id, $data) = @$msg;
             $self->{inbox}{$id}->put($data);
@@ -73,8 +67,7 @@ sub recv {
 
 sub readable {
     my $self = shift;
-    $self->{read_sem}->down;
-    #$self->{in}->readable;
+    $self->{in}->readable;
 }
 
 1;
